// (C) 1997,   Andreas Hirczy <ahi@sbox.tu-graz.ac.at> 


#include <stdlib.h>
#include "particle.hh"

// initialise static data members: time_step, default and deviaton of initial
// conditions
float Particle::time_step = 1.0/25.0;
float Particle::x_def  =  0.0;
float Particle::y_def  =  0.0;
float Particle::z_def  =  0.0;
float Particle::dx_def =  0.0;
float Particle::dy_def =  0.0;
float Particle::dz_def = 35.0;
float Particle::x_dev  =  0.0;
float Particle::y_dev  =  0.0;
float Particle::z_dev  =  0.0;
float Particle::dx_dev =  0.0;
float Particle::dy_dev =  0.0;
float Particle::dz_dev =  1.0;



Particle::Particle()
{
  // uncertainty of velocity is distrubuted evenly around standard velocity
  double phi    =  360.0 * rand()/(RAND_MAX+1.0);
  double theta  =   90.0 * rand()/(RAND_MAX+1.0);
  double radius = dz_dev * rand()/(RAND_MAX+1.0);

  x = y = z = 0.0;
  dx  = 0 + radius * sin(theta) * cos(phi);
  dy  = 0 + radius * sin(theta) * sin(phi);
  dz  = 35 + radius * cos(theta);
  ddx =  0.0;
  ddy =  0.0;
  ddz = -10.0;  // Gravity: ~ -10 m/s^2

  // and now we move this particle for a random fraction of a "time_step" to
  // avoid clustering of particles generated at the same moment
  float t = time_step * rand()/(RAND_MAX+1.0);
  dx = dx + ddx * t;
  dy = dy + ddy * t;
  dz = dz + ddz * t;
  x  =  x +  dx * t;
  y  =  y +  dy * t;
  z  =  z +  dz * t;
};

void Particle::Next(unsigned int nr_steps)
{
  for( unsigned int i=0; i<nr_steps; i++)
    {
      // euler integration of second order differential equation
      dx = dx + ddx * time_step;
      dy = dy + ddy * time_step;
      dz = dz + ddz * time_step;
      x  =  x +  dx * time_step;
      y  =  y +  dy * time_step;
      z  =  z +  dz * time_step;
    }
}



ostream& operator<<(ostream& s, Particle p)
{
  return s << p.X()  <<  '\t' << p.Y()  << '\t' << p.Z()  << '\t'
	   << p.dX() <<  '\t' << p.dY() << '\t' << p.dZ() << '\n';
};
